export default class ByteReader {
  private dvArray: DataView[];
  private dvIndex = 0;
  private pos = 0;
  constructor(dvArray: DataView[], startDvIndex: number, offset: number) {
    this.dvArray = dvArray;
    this.dvIndex = startDvIndex;
    this.pos = offset;
  }
  /** Reads one byte and returns it. */
  public readUint8() {
    const curByte = this.dvArray[this.dvIndex].getUint8(this.pos);
    this.pos += 1;
    if (this.pos >= this.dvArray[this.dvIndex].byteLength) {
      this.pos = 0;
      this.dvIndex += 1;
      if (this.dvIndex >= this.dvArray.length) { throw new Error('Tried to read beyond DataView boundary in extractFile'); }
    }
    return curByte;
  }
  /** Reads two bytes as an unsigned 16-bit integer and returns it. */
  public readUint16() {
    return this.readUint8() | (this.readUint8() << 8);
  }
  /** Reads four bytes as an unsigned 32-bit integer and returns it. */
  public readUint32() {
    return (this.readUint8() | (this.readUint8() << 8) | (this.readUint8() << 16) | (this.readUint8() << 24)) >>> 0;
  }
  /** Seeks the given number of bytes forward, without reading the bytes. */
  public seek(numBytes: number) {
    this.pos += numBytes;
    if (this.pos >= this.dvArray[this.dvIndex].byteLength) {
      this.pos -= this.dvArray[this.dvIndex].byteLength;
      this.dvIndex += 1;
      if (this.dvIndex >= this.dvArray.length) { throw new Error('Tried to read beyond DataView boundary in extractFile'); }
    }
  }
  /** Reads the given amount of bytes, and returns it as a DataView. */
  public extractDv(length: number) {
    const dv = new DataView(new ArrayBuffer(length));
    for (let i = 0; i < length; i += 1) {
        dv.setUint8(i, this.readUint8()); //TODO: refactor this so it is more optimized
    }
    return dv;
  }
}
