//Similar to extractFile.ts, but instead of receiving and returning an ArrayBuffer, works with Node.js streams.

import * as stream from 'stream';
import * as zlib from 'zlib';
import { ISsnFileEntry } from '../interfaces/ISsnFileEntry';
import decryptStream from './streams/decryptStream';
import streamSetMaxLength from './streams/streamSetMaxLength';

/** Extracts the file with the given metadata from the stream.
 * The stream must already start at the .zip's local file header
 * and must transparently span across multiple disks if necessary.
 */
export default function extractFileStream(file: ISsnFileEntry, inputStream: stream.Readable): stream.Readable {
  let curStream = inputStream;

  curStream = streamSetMaxLength(curStream, file.compressedSize);

  //pipe into decryption if file is encrypted
  if (file.decryptionKeys !== undefined) {
    const decryptTransform = decryptStream(file.decryptionKeys);
    curStream = curStream.pipe(decryptTransform);
  }

  //pipe into decompression
  const decompressTransform = zlib.createInflateRaw();
  decompressTransform.on('error', (error) => {
    throw new Error(`Error during decompression: ${error.message}`);
  });
  curStream = curStream.pipe(decompressTransform);

  return curStream;
}
