import * as stream from 'stream';

const BUFFER_SIZE = 16 * 1024;

export default function arrayBufferToStream(arrayBuffer: ArrayBuffer, offset: number = 0, length?: number): stream.Readable {
  if (offset < 0 || offset >= arrayBuffer.byteLength - 1) {
    throw new RangeError('Could not convert ArrayBuffer to ReadableStream; out of bounds.');
  }
  if (length !== undefined && offset + length >= arrayBuffer.byteLength - 1) {
    throw new RangeError('Could not convert ArrayBuffer to ReadableStream; out of bounds.');
  }

  let position = offset;
  const endPosition = (length !== undefined) ? (offset + length) : arrayBuffer.byteLength;
  const outStream = new stream.Readable({
    read(size) {
      const chunkSize = Math.min(size || BUFFER_SIZE, endPosition - position); //TODO: we can probably remove BUFFER_SIZE
      let needMoreData: boolean;
      do {
        //If end is reached
        if (position >= endPosition - 1) {
          this.push(null);
          return;
        }

        //Write chunk to stream
        const chunk = Buffer.from(arrayBuffer, position, chunkSize);
        position += chunk.length;
        needMoreData = this.push(chunk);
      } while (needMoreData);
    },
  });

  return outStream;
}
