import * as stream from 'stream';

export default function streamToArrayBuffer(inputStream: stream.Readable): Promise<ArrayBuffer> {
  return new Promise((resolve, reject) => {
    const chunks: Buffer[] = [];
    let totalSize = 0;

    //Convert chunks to string
    inputStream.on('data', (chunk: Buffer) => {
      chunks.push(chunk);
      totalSize += chunk.length;
    });

    //Output final string
    inputStream.on('end', () => {
      const outBuffer = Buffer.concat(chunks, totalSize);
      resolve(outBuffer.buffer as ArrayBuffer);
    });

    //Exit on error
    inputStream.on('error', (error) => {
      reject(error);
    });
  });
}
