import { Product } from '../../interfaces/ISettings';
import { ISsnFileEntry } from '../../interfaces/ISsnFileEntry';
import { SsnDiffType } from '../../interfaces/ISsnFileEntry';

/** Receives a list of file entries from the .zip file and checks them for correctness */
export default function verifyPatch(fileEntries: ISsnFileEntry[], product: Product, from: number): void {
  //There must be at least a .version file
  if (fileEntries.length === 0) {
    throw new Error('Expected at least a version file in the ');
  }

  //Check that last file is the .version file with diffType 0.
  const lastFile = fileEntries[fileEntries.length - 1];
  const validLastFileName = `${product}.version`;
  if (lastFile.name !== validLastFileName) {
    throw new Error(`Last file must be called "${validLastFileName}" but it was "${lastFile.name}".`);
  }
  if (lastFile.diffType !== SsnDiffType.NewFile) {
    throw new Error(`Last file (.version file) must have diffType 0 but it had diffType ${lastFile.diffType}.`);
  }

  //Patches from -1 must not have diff type NewFile
  if (from === -1) {
    fileEntries.filter((file) => file.diffType !== SsnDiffType.NewFile).forEach((file) => {
      throw new Error(`Patches from -1 must be included in full, but this patch had a file "${file.name}" with diff type ${file.diffType}.`);
    });
  }
}
